SystemOrganization addCategory: #'Spec-Debugger'!

!PreDebugWindow methodsFor: '*spec-debugger' stamp: 'AndreiChis 12/13/2012 12:25'!
spawnSpecDebugger
	model spawnSpecDebugger! !

Debugger subclass: #SpecProxyDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Debugger'!

!SpecProxyDebugger class methodsFor: 'class initialization' stamp: 'AndreiChis 12/13/2012 11:46'!
initialize
	super initialize.

	"Add ourselves to registry. See [Smalltalk tools]" 
	Smalltalk tools register: self as: #debugger! !

!SpecProxyDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2012 11:46'!
debuggerClass
	^ SpecDebugger ! !

!SpecProxyDebugger methodsFor: 'initialize' stamp: 'AndreiChis 12/13/2012 12:26'!
preDebugButtonQuads
	| buttons |
	self isPostMortem 
		ifTrue: [ ^ self postMortemPreDebugButtonQuads ].
	^ {
		{'Proceed' translated.	#proceed. 	#blue. 	'Continue execution' translated}.
		{'Abandon' translated.	#abandon. 	#black.	'Abandon this execution by closing this window' translated}.
		{'Debug'    		translated.		#debug.		#red. 	'Bring up a debugger' translated}.
		{'Spec Debug' translated. #spawnSpecDebugger. #red. 'Bring up the Spec Debugger' translated}
	}! !

!SpecProxyDebugger methodsFor: 'notifier menu' stamp: 'AndreiChis 12/13/2012 12:27'!
spawnSpecDebugger
	| topView |
	topView := self topView.
	topView model: nil.
	"so close won't release me."
	self breakDependents.
	topView delete.
	
	self debuggerClass debugContext: self interruptedContext fromProcess: self interruptedProcess! !

ComposableModel subclass: #SpecDebugger
	instanceVariableNames: 'model stack code toolbar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Debugger'!

!SpecDebugger class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 12/6/2012 18:58'!
debugContext: aContext fromProcess: aProcess

	<primitive: 19> "simulation guard"

	WorldState addDeferredUIMessage: [

		self new 
			model: (GTDebuggerSession process: aProcess context: aContext); 
			openWithSpec;
			ensureBehavior;
			yourself
		]
! !

!SpecDebugger class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 12/6/2012 16:34'!
spec
	<spec: #default>
	
	^ SpecLayout composed
		newColumn: [ :column | 
			column
				add: #stack;
				add: #toolbar height: 25;
				addSplitter;
				add: #code ]! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 12/9/2012 20:35'!
close

	self removeActionsForModel: self model.
	self model terminate.
	 ! !

!SpecDebugger methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/7/2012 17:31'!
code

	^ code! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 12/6/2012 16:21'!
columnsFor: anEntry
	| stream | 
	
	stream := ReadWriteStream on: ''.
	stream 
		nextPutAll: anEntry receiver class name asString.
	
	anEntry receiver class == anEntry methodClass
		ifFalse: [ 
			stream
				nextPutAll: '(';
				nextPutAll: anEntry methodClass name asString ;
				nextPutAll: ')'].

	"^ {stream contents . anEntry selector}"
	^ String streamContents: [:s | anEntry printDebugOn: s ]! !

!SpecDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 19:06'!
context

	^ self model context! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 12/9/2012 21:08'!
ensureBehavior

	self code behavior: self code behavior.
	self code setSelection: (self model selectedCodeRange: self stack selectedItem) ! !

!SpecDebugger methodsFor: 'protocol' stamp: 'AndreiChis 12/6/2012 18:08'!
initialExtent 

	^  410@510! !

!SpecDebugger methodsFor: 'initialization' stamp: 'AndreiChis 12/7/2012 18:07'!
initialize

	model := nil asValueHolder.
	super initialize.
	
	model whenChangedDo: [:newModel :oldModel |
		self removeActionsForModel: oldModel.
		self registerActionsForModel: newModel.
		self updateToolbarFromModel: newModel.
		self updateStackFromModel: newModel ].! !

!SpecDebugger methodsFor: 'initialization' stamp: 'AndreiChis 12/9/2012 21:01'!
initializePresenter
	
	self stack whenListChanged: [
		self stack setSelectedItem: self context ].

	self stack whenSelectedItemChanged: [:aContext |
		self toolbar selectedContext: aContext.
		self updateCodeFromContext: aContext ]! !

!SpecDebugger methodsFor: 'initialization' stamp: 'AndreiChis 12/7/2012 18:02'!
initializeWidgets 

	self instantiateModels: {
		#stack -> #ListComposableModel. 
		#code -> #TextModel.
		#toolbar -> #SpecDebuggerToolbar }.

	self stack displayBlock: [:e | self columnsFor: e ].
	self code aboutToStyle: true.! !

!SpecDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:00'!
model

	^ model contents! !

!SpecDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 15:59'!
model: aModel

	model contents: aModel ! !

!SpecDebugger methodsFor: 'protocol' stamp: 'AndreiChis 12/6/2012 17:56'!
openWithSpec 

	| w |
	w := super openWithSpec.
	w whenClosedDo: [ self close ].
	^ w! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 12/7/2012 17:56'!
registerActionsForModel: aModel

	aModel ifNotNil: [
		aModel
			when: #restart send: #updateRestart to: self;
			when: #resume send: #updateResume to: self;
			when: #stepInto send: #updateStepInto to: self;
			when: #stepOver send: #updateStepOver to: self;
			when: #stepThrough send: #updateStepThrough to: self ]! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 12/7/2012 17:56'!
removeActionsForModel: aModel

	aModel ifNotNil: [ 
		aModel  
			removeActionsForEvent: #restart;
			removeActionsForEvent: #resume;
			removeActionsForEvent: #stepInto;
			removeActionsForEvent: #stepOver;
			removeActionsForEvent: #stepThrough ]. ! !

!SpecDebugger methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 21:05'!
stack

	^ stack! !

!SpecDebugger methodsFor: 'protocol' stamp: 'AndreiChis 12/6/2012 18:07'!
title

	^ 'Spec Debugger'! !

!SpecDebugger methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 21:05'!
toolbar

	^ toolbar! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 12/7/2012 18:05'!
updateCodeFromContext: aContext 

	aContext 
		ifNil: [
			self code 
				text: '';
				behavior: nil ]
		ifNotNil: [
			self code
				text: aContext sourceCode;
				behavior: aContext receiver class;
				setSelection: (self model selectedCodeRange: aContext) ]! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/7/2012 17:27'!
updateRestart

	self updateStep 
	! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/7/2012 17:26'!
updateResume

	! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 12/7/2012 18:01'!
updateStackFromModel: aModel

	self stack items: aModel stack! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 12/7/2012 18:01'!
updateStep

	self updateStackFromModel: self model.
	
! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/6/2012 17:05'!
updateStepInto

	self updateStep ! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/6/2012 17:04'!
updateStepOver

	self updateStep 
	! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/7/2012 17:26'!
updateStepThrough

	self updateStep ! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 12/7/2012 18:02'!
updateToolbarFromModel: aModel

	self toolbar model: aModel.! !

ComposableModel subclass: #SpecDebuggerToolbar
	instanceVariableNames: 'model selectedContext resumeButton restartButton stepIntoButton stepOverButton stepThroughButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Debugger'!

!SpecDebuggerToolbar class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 12/9/2012 20:57'!
spec
	<spec>
	
	^ SpecLayout composed
		newRow: [ :r | 
			r
				add: #resumeButton;
				add: #restartButton;
				add: #stepIntoButton;
				add: #stepOverButton;
				add: #stepThroughButton ]! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/9/2012 20:42'!
currentContext

	^ self selectedContext isNil 
		ifTrue: [ self model context ]
		ifFalse: [ self selectedContext ]! !

!SpecDebuggerToolbar methodsFor: 'initialization' stamp: 'AndreiChis 12/6/2012 16:41'!
initialize

	selectedContext := nil asValueHolder.
	model := nil asValueHolder.
	super initialize! !

!SpecDebuggerToolbar methodsFor: 'initialization' stamp: 'AndreiChis 12/9/2012 20:59'!
initializeWidgets 

	self instantiateModels: {
		#resumeButton -> #ButtonModel.
		#restartButton -> #ButtonModel.
		#stepIntoButton -> #ButtonModel.
		#stepOverButton -> #ButtonModel.
		#stepThroughButton -> #ButtonModel }.
		
	self resumeButton
		label: 'Proceed';
		state: false;
		action: [ self resumeAction ].
	self restartButton
		label: 'Restart';
		state: false;
		action: [ self restartAction ].
	self stepIntoButton
		label: 'Into';
		state: false;
		action: [ self stepIntoAction ].
	self stepOverButton
		label: 'Over';
		state: false;
		action: [ self stepOverAction ].
	self stepThroughButton  
		label: 'Through';
		state: false;
		action: [ self stepThroughAction ]! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:41'!
model
	
	^ model contents ! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:41'!
model: aModel
	
	model contents: aModel! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 12/9/2012 20:46'!
restartAction

	self model restart: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:55'!
restartButton

	^ restartButton! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 12/9/2012 20:59'!
resumeAction

	self model 
		resume;
		clear.
	self window delete.! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:55'!
resumeButton

	^ resumeButton! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:39'!
selectedContext
	
	^ selectedContext contents ! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:40'!
selectedContext: aContext
	
	selectedContext contents: aContext! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 12/9/2012 20:47'!
stepIntoAction

	self model stepInto: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:55'!
stepIntoButton

	^ stepIntoButton! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 12/9/2012 20:47'!
stepOverAction

	self model stepOver: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:56'!
stepOverButton

	^ stepOverButton ! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 12/9/2012 20:47'!
stepThroughAction

	self model stepThrough: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:56'!
stepThroughButton

	^ stepThroughButton ! !
SpecProxyDebugger initialize!
